/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.transform;

import java.lang.reflect.Constructor;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.language.executable.ExecutableBase;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.exception.TransformationFailedException;

/**
 * Base class for transformations from {@link RaspberryPi} model elements to executables.
 * 
 * @author hoelzl
 */
public class RasPiGeneratorTransformationBase implements ITransformationProvider {
	/** The model element class. */
	private final Class<? extends EObject> modelElementClass;
	/** The executable class. */
	private final Class<? extends ExecutableBase<? extends EObject>> executableClass;

	/** Constructor. */
	protected RasPiGeneratorTransformationBase(Class<? extends EObject> modelElementClass,
			Class<? extends ExecutableBase<? extends EObject>> executableClass) {
		this.modelElementClass = modelElementClass;
		this.executableClass = executableClass;
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getTargetClass() {
		return executableClass;
	}

	/** {@inheritDoc} */
	@Override
	public boolean
			canHandleChainTransformation(Class<?> sourceClass, ITransformationContext context) {
		return modelElementClass.isAssignableFrom(sourceClass);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canTransform(Object source, ITransformationContext context) {
		return source != null && modelElementClass.isInstance(source);
	}

	/** {@inheritDoc} */
	@Override
	public Object transform(Object source, ITransformationContext context)
			throws TransformationFailedException {
		try {
			Constructor<? extends ExecutableBase<? extends EObject>> c =
					executableClass.getConstructor(modelElementClass);
			return c.newInstance(source);
		} catch(Exception e) {
			e.printStackTrace(System.err);
			throw new TransformationFailedException(this, null, null);
		}
	}
}
